@@ -35,7 +35,7 @@
 #include "choose.h"
 #include "gdm-net.h"
 
-static const gchar RCSid[]="$Id: display.c,v 1.21 2001/09/01 09:06:30 jirka Exp $";
+static const gchar RCSid[]="$Id: display.c,v 1.22 2001/09/11 03:26:01 jirka Exp $";
 
 /* External vars */
 extern gboolean GdmXdmcp;
@@ -157,7 +157,8 @@ gdm_display_manage (GdmDisplay *d)
     }
 
     /* Fork slave process */
-    switch (d->slavepid = fork()) {
+    gdm_safe_fork (&(d->slavepid));
+    switch (d->slavepid) {
 
     case 0:
 	setpgid (0, 0);
@@ -211,7 +212,8 @@ gdm_display_manage (GdmDisplay *d)
 	return FALSE;
 
     default:
-	gdm_debug ("gdm_display_manage: Forked slave: %d", d->slavepid);
+	gdm_debug ("gdm_display_manage: Forked slave: %d",
+		   (int)d->slavepid);
 	break;
     }
 
@@ -300,6 +302,9 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->authfile);
     d->authfile = NULL;
 
+    g_free (d->authfile_gdm);
+    d->authfile_gdm = NULL;
+
     if (d->auths) {
 	GSList *tmpauth = d->auths;
 
