@@ -31,7 +31,7 @@
 #include "verify.h"
 #include "errorgui.h"
 
-static const gchar RCSid[]="$Id: verify-pam.c,v 1.21 2001/09/27 04:10:06 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-pam.c,v 1.22 2001/10/05 20:55:07 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -664,7 +664,7 @@ gdm_verify_open_session (GdmDisplay *d)
 	}
 
 	/* Set credentials */
-	if ((pamerr = pam_setcred (pamh, 0)) != PAM_SUCCESS) {
+	if ((pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED)) != PAM_SUCCESS) {
 		if (gdm_slave_should_complain ())
 			gdm_error (_("Couldn't set credentials for %s"),
 				   current_login);
