@@ -23,7 +23,7 @@
 #include <shadow.h>
 
 #ifdef HAVE_CRYPT
-  #include <crypt.h>
+#  include <crypt.h>
 #endif /* HAVE_CRYPT */
 
 #include <vicious.h>
@@ -33,7 +33,7 @@
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-shadow.c,v 1.17 2001/08/22 23:11:54 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-shadow.c,v 1.18 2001/09/27 04:10:06 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -54,7 +54,7 @@ extern gboolean GdmAllowRemoteRoot;
  */
 
 gchar *
-gdm_verify_user (const char *username, const gchar *display, gboolean local) 
+gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboolean local) 
 {
     gchar *login, *passwd, *ppasswd, *auth_errmsg;
     struct passwd *pwent;
@@ -211,7 +211,7 @@ gdm_verify_user (const char *username, const gchar *display, gboolean local)
  */
 
 void
-gdm_verify_setup_user (const gchar *login, const gchar *display) 
+gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display) 
 {
 }
 
@@ -222,7 +222,7 @@ gdm_verify_setup_user (const gchar *login, const gchar *display)
  * Unregister the user's session */
 
 void
-gdm_verify_cleanup (void)
+gdm_verify_cleanup (GdmDisplay *d)
 {
 }
 
@@ -242,7 +242,7 @@ gdm_verify_check (void)
 
 /* used in pam */
 gboolean
-gdm_verify_open_session (void)
+gdm_verify_open_session (GdmDisplay *d)
 {
 	return TRUE;
 }
